@@ -4,6 +4,10 @@
 #include "acodec.h"
 
 
+/* globals */
+static bool acodec_inited = false;
+
+
 /* Function: al_get_allegro_acodec_version
  */
 uint32_t al_get_allegro_acodec_version(void)
@@ -38,14 +42,7 @@ bool al_init_acodec_addon(void)
 #endif
 
 #ifdef ALLEGRO_CFG_ACODEC_MODAUDIO
-   ret &= al_register_audio_stream_loader(".xm", _al_load_xm_audio_stream);
-   ret &= al_register_audio_stream_loader_f(".xm", _al_load_xm_audio_stream_f);
-   ret &= al_register_audio_stream_loader(".it", _al_load_it_audio_stream);
-   ret &= al_register_audio_stream_loader_f(".it", _al_load_it_audio_stream_f);
-   ret &= al_register_audio_stream_loader(".mod", _al_load_mod_audio_stream);
-   ret &= al_register_audio_stream_loader_f(".mod", _al_load_mod_audio_stream_f);
-   ret &= al_register_audio_stream_loader(".s3m", _al_load_s3m_audio_stream);
-   ret &= al_register_audio_stream_loader_f(".s3m", _al_load_s3m_audio_stream_f);
+   ret &= _al_register_dumb_loaders();
 #endif
 
 #ifdef ALLEGRO_CFG_ACODEC_VORBIS
@@ -62,8 +59,25 @@ bool al_init_acodec_addon(void)
    ret &= al_register_audio_stream_loader_f(".opus", _al_load_ogg_opus_audio_stream_f);
 #endif
 
+#ifdef ALLEGRO_CFG_ACODEC_MP3
+   ret &= al_register_sample_loader(".mp3", _al_load_mp3);
+   ret &= al_register_audio_stream_loader(".mp3", _al_load_mp3_audio_stream);
+   ret &= al_register_sample_loader_f(".mp3", _al_load_mp3_f);
+   ret &= al_register_audio_stream_loader_f(".mp3", _al_load_mp3_audio_stream_f);
+#endif
+
+   acodec_inited = ret;
+
    return ret;
 }
 
 
+/* Function: al_is_acodec_addon_initialized
+ */
+bool al_is_acodec_addon_initialized(void)
+{
+   return acodec_inited;
+}
+
+
 /* vim: set sts=3 sw=3 et: */
