@@ -6,6 +6,7 @@
 #include "allegro5/internal/aintern_native_dialog_cfg.h"
 #include "allegro5/internal/aintern_xdisplay.h"
 #include "gtk_dialog.h"
+#include "gtk_xgtk.h"
 
 ALLEGRO_DEBUG_CHANNEL("gtk_dialog")
 
@@ -19,12 +20,13 @@ bool _al_init_native_dialog_addon(void)
       return false;
    }
 
-   return true;
+   return _al_gtk_set_display_overridable_interface(true);
 }
 
 
 void _al_shutdown_native_dialog_addon(void)
 {
+   _al_gtk_set_display_overridable_interface(false);
 }
 
 
