@@ -33,6 +33,7 @@ static gboolean create_gtk_file_dialog(gpointer data)
    ALLEGRO_DISPLAY *display = msg->display;
    ALLEGRO_NATIVE_DIALOG *fd = msg->dialog;
    bool save = fd->flags & ALLEGRO_FILECHOOSER_SAVE;
+   bool folder = fd->flags & ALLEGRO_FILECHOOSER_FOLDER;
    gint result;
 
    GtkWidget *window;
@@ -40,15 +41,58 @@ static gboolean create_gtk_file_dialog(gpointer data)
    window =
       gtk_file_chooser_dialog_new(al_cstr(fd->title),
                                   NULL,
-                                  save ? GTK_FILE_CHOOSER_ACTION_SAVE : GTK_FILE_CHOOSER_ACTION_OPEN,
+                                  save ? GTK_FILE_CHOOSER_ACTION_SAVE : folder ? GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER : GTK_FILE_CHOOSER_ACTION_OPEN,
                                   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                   save ? GTK_STOCK_SAVE : GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
 
    _al_gtk_make_transient(display, window);
 
+   if (save) {
+      gtk_file_chooser_set_do_overwrite_confirmation
+         (GTK_FILE_CHOOSER(window), true);
+   }
+
    if (fd->fc_initial_path) {
-      gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(window),
-         al_path_cstr(fd->fc_initial_path, ALLEGRO_NATIVE_PATH_SEP));
+      bool is_dir;
+      bool exists;
+      const char *path = al_path_cstr(fd->fc_initial_path, ALLEGRO_NATIVE_PATH_SEP);
+
+      if (al_filename_exists(path)) {
+         exists = true;
+         ALLEGRO_FS_ENTRY *fs = al_create_fs_entry(path);
+         is_dir = al_get_fs_entry_mode(fs) & ALLEGRO_FILEMODE_ISDIR;
+         al_destroy_fs_entry(fs);
+      }
+      else {
+         exists = false;
+         is_dir = false;
+      }
+
+      if (is_dir) {
+         gtk_file_chooser_set_current_folder
+            (GTK_FILE_CHOOSER(window),
+             al_path_cstr(fd->fc_initial_path, ALLEGRO_NATIVE_PATH_SEP));
+      }
+      else if (exists) {
+         gtk_file_chooser_set_filename
+             (GTK_FILE_CHOOSER(window),
+              al_path_cstr(fd->fc_initial_path, ALLEGRO_NATIVE_PATH_SEP));
+      }
+      else {
+         ALLEGRO_PATH *dir_path = al_clone_path(fd->fc_initial_path);
+         if (dir_path) {
+            al_set_path_filename(dir_path, NULL);
+            gtk_file_chooser_set_current_folder
+               (GTK_FILE_CHOOSER(window),
+                al_path_cstr(dir_path, ALLEGRO_NATIVE_PATH_SEP));
+            if (save) {
+               gtk_file_chooser_set_current_name
+                  (GTK_FILE_CHOOSER(window),
+                   al_get_path_filename(fd->fc_initial_path));
+            }
+            al_destroy_path(dir_path);
+         }
+      }
    }
 
    if (fd->flags & ALLEGRO_FILECHOOSER_MULTIPLE)
