@@ -80,7 +80,7 @@ ALLEGRO_TEXTLOG *al_open_native_text_log(char const *title, int flags)
       return NULL;
    }
 
-   _al_register_destructor(_al_dtor_list, "textlog", textlog,
+   textlog->dtor_item = _al_register_destructor(_al_dtor_list, "textlog", textlog,
       (void (*)(void *))al_close_native_text_log);
 
    return (ALLEGRO_TEXTLOG *)textlog;
@@ -111,7 +111,7 @@ void al_close_native_text_log(ALLEGRO_TEXTLOG *textlog)
          al_lock_mutex(dialog->tl_text_mutex);
       }
 
-      _al_unregister_destructor(_al_dtor_list, dialog);
+      _al_unregister_destructor(_al_dtor_list, dialog->dtor_item);
    }
 
    al_ustr_free(dialog->title);
