@@ -1,7 +1,3 @@
-#ifndef __al_included_allegro_aintern_video_h
-#define __al_included_allegro_aintern_video_h
-
-#include "allegro5/internal/aintern_list.h"
 
 typedef struct ALLEGRO_VIDEO_INTERFACE {
    bool (*open_video)(ALLEGRO_VIDEO *video);
@@ -14,7 +10,7 @@ typedef struct ALLEGRO_VIDEO_INTERFACE {
 
 struct ALLEGRO_VIDEO {
    ALLEGRO_VIDEO_INTERFACE *vtable;
-
+   
    /* video */
    ALLEGRO_BITMAP *current_frame;
    double video_position;
@@ -36,17 +32,9 @@ struct ALLEGRO_VIDEO {
    bool playing;
    double position;
 
-   _AL_LIST_ITEM *dtor_item;
-
    /* implementation specific */
    void *data;
 };
 
-typedef bool (*ALLEGRO_VIDEO_IDENTIFIER_FUNCTION)(ALLEGRO_FILE *f);
-
-void _al_compute_scaled_dimensions(int frame_w, int frame_h, float aspect_ratio, float *scaled_w, float *scaled_h);
-
 ALLEGRO_VIDEO_INTERFACE *_al_video_ogv_vtable(void);
-bool _al_video_identify_ogv(ALLEGRO_FILE *f);
-
-#endif
+void _al_compute_scaled_dimensions(int frame_w, int frame_h, float aspect_ratio, float *scaled_w, float *scaled_h);
