@@ -28,7 +28,7 @@
 
 typedef void (*Screen)(int);
 int ScreenW = 800, ScreenH = 600;
-#define NUM_SCREENS 10
+#define NUM_SCREENS 12
 #define ROTATE_SPEED 0.0001f
 Screen Screens[NUM_SCREENS];
 const char *ScreenName[NUM_SCREENS];
@@ -49,7 +49,8 @@ ALLEGRO_TRANSFORM MainTrans;
 enum MODE {
    INIT,
    LOGIC,
-   DRAW
+   DRAW,
+   DEINIT
 };
 
 typedef struct
@@ -480,12 +481,171 @@ static void IndexedPrimitives(int mode)
    }
 }
 
-int main(void)
+static void VertexBuffers(int mode)
+{
+   static ALLEGRO_VERTEX vtx[13];
+   static ALLEGRO_VERTEX vtx2[13];
+   static ALLEGRO_VERTEX_BUFFER* vbuff;
+   static ALLEGRO_VERTEX_BUFFER* vbuff2;
+   static bool no_soft;
+   static bool no_soft2;
+   if (mode == INIT) {
+      int ii = 0;
+      ALLEGRO_COLOR color;
+      for (ii = 0; ii < 13; ii++) {
+         float x, y;
+         x = 200 * cosf((float)ii / 13.0f * 2 * ALLEGRO_PI);
+         y = 200 * sinf((float)ii / 13.0f * 2 * ALLEGRO_PI);
+
+         color = al_map_rgb((ii + 1) % 3 * 64, (ii + 2) % 3 * 64, (ii) % 3 * 64);
+
+         vtx[ii].x = x; vtx[ii].y = y; vtx[ii].z = 0;
+         vtx2[ii].x = 0.1 * x; vtx2[ii].y = 0.1 * y;
+         vtx[ii].color = color;
+         vtx2[ii].color = color;
+      }
+      vbuff = al_create_vertex_buffer(0, vtx, 13, ALLEGRO_PRIM_BUFFER_READWRITE);
+      if (!vbuff) {
+         vbuff = al_create_vertex_buffer(0, vtx, 13, 0);
+         no_soft = true;
+      }
+      else {
+         no_soft = false;
+      }
+
+      vbuff2 = al_create_vertex_buffer(0, vtx2, 13, ALLEGRO_PRIM_BUFFER_READWRITE);
+      if (!vbuff2) {
+         vbuff2 = al_create_vertex_buffer(0, vtx2, 13, 0);
+         no_soft2 = true;
+      }
+      else {
+         no_soft2 = false;
+      }
+   } else if (mode == LOGIC) {
+      Theta += Speed;
+      al_build_transform(&MainTrans, ScreenW / 2, ScreenH / 2, 1, 1, Theta);
+   } else if (mode == DRAW) {
+      if (Blend)
+         al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ONE);
+      else
+         al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO);
+
+      al_use_transform(&MainTrans);
+
+      if (vbuff && !(Soft && no_soft)) {
+         al_draw_vertex_buffer(vbuff, 0, 0, 4, ALLEGRO_PRIM_LINE_LIST);
+         al_draw_vertex_buffer(vbuff, 0, 4, 9, ALLEGRO_PRIM_LINE_STRIP);
+         al_draw_vertex_buffer(vbuff, 0, 9, 13, ALLEGRO_PRIM_LINE_LOOP);
+      }
+      else {
+         al_draw_text(Font, al_map_rgb_f(1, 1, 1), 0, -40, 0, "Vertex buffers not supported");
+      }
+
+      if (vbuff2 && !(Soft && no_soft2)) {
+         al_draw_vertex_buffer(vbuff2, 0, 0, 13, ALLEGRO_PRIM_POINT_LIST);
+      }
+      else {
+         al_draw_text(Font, al_map_rgb_f(1, 1, 1), 0, 40, 0, "Vertex buffers not supported");
+      }
+
+      al_use_transform(&Identity);
+   } else if (mode == DEINIT) {
+      al_destroy_vertex_buffer(vbuff);
+      al_destroy_vertex_buffer(vbuff2);
+   }
+}
+
+static void IndexedBuffers(int mode)
+{
+   static ALLEGRO_VERTEX_BUFFER* vbuff;
+   static ALLEGRO_INDEX_BUFFER* ibuff;
+   static bool soft = true;
+   if (mode == INIT) {
+      int ii;
+      ALLEGRO_COLOR color;
+      ALLEGRO_VERTEX* vtx;
+      int flags = ALLEGRO_PRIM_BUFFER_READWRITE;
+
+      vbuff = al_create_vertex_buffer(NULL, NULL, 13, ALLEGRO_PRIM_BUFFER_READWRITE);
+      if (vbuff == NULL) {
+         vbuff = al_create_vertex_buffer(NULL, NULL, 13, 0);
+         soft = false;
+         flags = 0;
+      }
+
+      ibuff = al_create_index_buffer(sizeof(short), NULL, 8, flags);
+
+      if (vbuff) {
+         vtx = al_lock_vertex_buffer(vbuff, 0, 13, ALLEGRO_LOCK_WRITEONLY);
+
+         for (ii = 0; ii < 13; ii++) {
+            float x, y;
+            x = 200 * cosf((float)ii / 13.0f * 2 * ALLEGRO_PI);
+            y = 200 * sinf((float)ii / 13.0f * 2 * ALLEGRO_PI);
+
+            color = al_map_rgb((ii + 1) % 3 * 64, (ii + 2) % 3 * 64, (ii) % 3 * 64);
+
+            vtx[ii].x = x;
+            vtx[ii].y = y;
+            vtx[ii].z = 0;
+            vtx[ii].color = color;
+         }
+
+         al_unlock_vertex_buffer(vbuff);
+      }
+   } else if (mode == LOGIC) {
+      if (ibuff) {
+         int ii;
+         int t = (int)al_get_time();
+         short* indices = al_lock_index_buffer(ibuff, 0, 8, ALLEGRO_LOCK_WRITEONLY);
+
+         for (ii = 0; ii < 8; ii++) {
+            indices[ii] = (t + ii) % 13;
+         }
+         al_unlock_index_buffer(ibuff);
+      }
+
+      Theta += Speed;
+      al_build_transform(&MainTrans, ScreenW / 2, ScreenH / 2, 1, 1, Theta);
+   } else if (mode == DRAW) {
+      if (Blend)
+         al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ONE);
+      else
+         al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO);
+
+      al_use_transform(&MainTrans);
+
+      if(!(Soft && !soft) && vbuff && ibuff) {
+         al_draw_indexed_buffer(vbuff, NULL, ibuff, 0, 4, ALLEGRO_PRIM_LINE_LIST);
+         al_draw_indexed_buffer(vbuff, NULL, ibuff, 4, 8, ALLEGRO_PRIM_LINE_STRIP);
+      }
+      else {
+         al_draw_text(Font, al_map_rgb_f(1, 1, 1), 0, 0, 0, "Indexed buffers not supported");
+      }
+
+      al_use_transform(&Identity);
+   } else if (mode == DEINIT) {
+      al_destroy_vertex_buffer(vbuff);
+      al_destroy_index_buffer(ibuff);
+   }
+}
+
+int main(int argc, char **argv)
 {
    ALLEGRO_DISPLAY *display;
    ALLEGRO_BITMAP* bkg;
    ALLEGRO_COLOR black;
    ALLEGRO_EVENT_QUEUE *queue;
+   bool use_shader = false;
+
+   if (argc > 1) {
+      if (strcmp(argv[1], "--shader") == 0) {
+         use_shader = true;
+      }
+      else {
+         abort_example("Invalid command line option: %s", argv[1]);
+      }
+   }
 
    // Initialize Allegro 5 and addons
    if (!al_init()) {
@@ -494,13 +654,18 @@ int main(void)
    al_init_image_addon();
    al_init_font_addon();
    al_init_primitives_addon();
+   init_platform_specific();
    
+   if (use_shader) {
+      al_set_new_display_flags(ALLEGRO_PROGRAMMABLE_PIPELINE);
+   }
+
    // Create a window to display things on: 640x480 pixels
    display = al_create_display(ScreenW, ScreenH);
    if (!display) {
       abort_example("Error creating display.\n");
    }
-   
+
    // Install the keyboard handler
    if (!al_install_keyboard()) {
       abort_example("Error installing keyboard.\n");
@@ -570,6 +735,8 @@ int main(void)
    Screens[7] = TexturePrimitives;
    Screens[8] = FilledTexturePrimitives;
    Screens[9] = CustomVertexFormatPrimitives;
+   Screens[10] = VertexBuffers;
+   Screens[11] = IndexedBuffers;
 
    ScreenName[0] = "Low Level Primitives";
    ScreenName[1] = "Indexed Primitives";
@@ -581,10 +748,14 @@ int main(void)
    ScreenName[7] = "Textured Primitives";
    ScreenName[8] = "Filled Textured Primitives";
    ScreenName[9] = "Custom Vertex Format";
+   ScreenName[10] = "Vertex Buffers";
+   ScreenName[11] = "Indexed Buffers";
 
-   for (ii = 0; ii < NUM_SCREENS; ii++)
+   for (ii = 0; ii < NUM_SCREENS; ii++) {
       Screens[ii](INIT);
-      
+      Screens[ii](LOGIC);
+   }
+
    while (!done) {
       double frame_duration = al_get_time() - real_time;
       al_rest(fixed_timestep - frame_duration); //rest at least fixed_dt
@@ -711,7 +882,7 @@ int main(void)
       }
       
       Screens[cur_screen](DRAW);
-      
+
       al_set_clipping_rectangle(0, 0, ScreenW, ScreenH);
 
       if (Soft == 1) {
@@ -734,7 +905,11 @@ int main(void)
       al_flip_display();
       frames_done++;
    }
+
+   for (ii = 0; ii < NUM_SCREENS; ii++)
+      Screens[ii](DEINIT);
+
    }
-   
+
    return 0;
 }
