@@ -9,7 +9,7 @@
 
 #include "common.c"
 
-int main(void)
+int main(int argc, char **argv)
 {
    const int display_w = 640;
    const int display_h = 480;
@@ -33,12 +33,16 @@ int main(void)
    bool clip_mode = false;
    ALLEGRO_COLOR tint;
 
+   (void)argc;
+   (void)argv;
+
    if (!al_init()) {
       abort_example("Could not init Allegro.\n");
    }
 
    al_install_keyboard();
    al_init_image_addon();
+   init_platform_specific();
 
    open_log();
    log_printf("Press 'w' to toggle wide mode.\n");
