@@ -6,8 +6,6 @@
 ALLEGRO_KEYBOARD_DRIVER *_al_xwin_keyboard_driver(void);
 void _al_xwin_keyboard_handler(XKeyEvent *event, ALLEGRO_DISPLAY *display);
 void _al_xwin_keyboard_switch_handler(ALLEGRO_DISPLAY *display, bool focus_in);
-void _al_xwin_keyboard_handler_alternative(bool press, int hardware_keycode,
-   uint32_t unichar, ALLEGRO_DISPLAY *display);
 
 #endif
 
