@@ -1,8 +1,8 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python3
 #
 # Generate the routines to draw each scanline of a primitive.
 # Run:
-#  python2 misc/make_scanline_drawers.py | indent -kr -i3 -l0 > src/scanline_drawers.inc
+#  misc/make_scanline_drawers.py | indent -kr -i3 -l0 > src/scanline_drawers.inc
 
 import sys, re
 
@@ -30,38 +30,38 @@ def make_drawer(name):
    if shade and opaque:
       raise Exception("shade and opaque")
 
-   print interp("static void #{name} (uintptr_t state, int x1, int y, int x2) {")
+   print(interp("static void #{name} (uintptr_t state, int x1, int y, int x2) {"))
 
    if not texture:
       if grad:
-         print """\
+         print("""\
          state_grad_any_2d *gs = (state_grad_any_2d *)state;
          state_solid_any_2d *s = &gs->solid;
          ALLEGRO_COLOR cur_color = s->cur_color;
-         """
+         """)
       else:
-         print """\
+         print("""\
          state_solid_any_2d *s = (state_solid_any_2d *)state;
          ALLEGRO_COLOR cur_color = s->cur_color;
-         """
+         """)
    else:
       if grad:
-         print """\
+         print("""\
          state_texture_grad_any_2d *gs = (state_texture_grad_any_2d *)state;
          state_texture_solid_any_2d *s = &gs->solid;
          ALLEGRO_COLOR cur_color = s->cur_color;
-         """
+         """)
       else:
-         print """\
+         print("""\
          state_texture_solid_any_2d *s = (state_texture_solid_any_2d *)state;
-         """
-      print """\
+         """)
+      print("""\
          float u = s->u;
          float v = s->v;
-         """
+         """)
 
    # XXX still don't understand why y-1 is required
-   print """\
+   print("""\
       ALLEGRO_BITMAP *target = s->target;
 
       if (target->parent) {
@@ -81,44 +81,44 @@ def make_drawer(name):
       }
 
       if (x1 < 0) {
-      """
+      """)
    if texture:
-      print """\
+      print("""\
          u += s->du_dx * -x1;
          v += s->dv_dx * -x1;
-         """
+         """)
 
    if grad:
-      print """\
+      print("""\
          cur_color.r += gs->color_dx.r * -x1;
          cur_color.g += gs->color_dx.g * -x1;
          cur_color.b += gs->color_dx.b * -x1;
          cur_color.a += gs->color_dx.a * -x1;
-         """
+         """)
 
-   print """\
+   print("""\
          x1 = 0;
       }
 
       if (x2 > target->lock_w - 1) {
          x2 = target->lock_w - 1;
       }
-      """
+      """)
 
-   print "{"
+   print("{")
    if shade:
-      print """\
+      print("""\
       int op, src_mode, dst_mode;
       int op_alpha, src_alpha, dst_alpha;
       ALLEGRO_COLOR const_color;
-      al_get_separate_blender(&op, &src_mode, &dst_mode,
+      al_get_separate_bitmap_blender(&op, &src_mode, &dst_mode,
          &op_alpha, &src_alpha, &dst_alpha);
       const_color = al_get_blend_color();
-      """
+      """)
 
-   print "{"
+   print("{")
    if texture:
-      print """\
+      print("""\
       const int offset_x = s->texture->parent ? s->texture->xofs : 0;
       const int offset_y = s->texture->parent ? s->texture->yofs : 0;
       ALLEGRO_BITMAP* texture = s->texture->parent ? s->texture->parent : s->texture;
@@ -132,15 +132,15 @@ def make_drawer(name):
       v = fmodf(v, s->h);
       ASSERT(0 <= u); ASSERT(u < s->w);
       ASSERT(0 <= v); ASSERT(v < s->h);
-      """
+      """)
 
-   print "{"
-   print """\
+   print("{")
+   print("""\
       const int dst_format = target->locked_region.format;
       uint8_t *dst_data = (uint8_t *)target->lock_data
          + y * target->locked_region.pitch
          + x1 * target->locked_region.pixel_size;
-      """
+      """)
 
    if shade:
       make_if_blender_loop(
@@ -154,7 +154,7 @@ def make_drawer(name):
             if_format='ALLEGRO_PIXEL_FORMAT_ARGB_8888',
             alpha_only=True
             )
-      print "else"
+      print("else")
       make_if_blender_loop(
             op='ALLEGRO_ADD',
             src_mode='ALLEGRO_ALPHA',
@@ -166,7 +166,7 @@ def make_drawer(name):
             if_format='ALLEGRO_PIXEL_FORMAT_ARGB_8888',
             alpha_only=True
             )
-      print "else"
+      print("else")
       make_if_blender_loop(
             op='ALLEGRO_ADD',
             src_mode='ALLEGRO_ONE',
@@ -178,29 +178,29 @@ def make_drawer(name):
             if_format='ALLEGRO_PIXEL_FORMAT_ARGB_8888',
             alpha_only=True
             )
-      print "else"
+      print("else")
 
    if opaque and white:
       make_loop(copy_format=True, src_size='4')
-      print "else"
+      print("else")
       make_loop(copy_format=True, src_size='3')
-      print "else"
+      print("else")
       make_loop(copy_format=True, src_size='2')
-      print "else"
+      print("else")
    else:
       make_loop(
             if_format='ALLEGRO_PIXEL_FORMAT_ARGB_8888'
             )
-      print "else"
+      print("else")
 
    make_loop()
 
-   print """\
+   print("""\
    }
    }
    }
    }
-   """
+   """)
 
 def make_if_blender_loop(
       op='op',
@@ -215,14 +215,14 @@ def make_if_blender_loop(
       if_format=None,
       alpha_only=False
       ):
-   print interp("""\
+   print(interp("""\
       if (op == #{op} &&
             src_mode == #{src_mode} &&
             src_alpha == #{src_alpha} &&
             op_alpha == #{op_alpha} &&
             dst_mode == #{dst_mode} &&
             dst_alpha == #{dst_alpha}) {
-      """)
+      """))
 
    if texture and if_format:
       make_loop(
@@ -236,7 +236,7 @@ def make_if_blender_loop(
             if_format=if_format,
             alpha_only=alpha_only
             )
-      print "else"
+      print("else")
 
    make_loop(
       op=op,
@@ -248,7 +248,7 @@ def make_if_blender_loop(
       const_color=const_color,
       alpha_only=alpha_only)
 
-   print "}"
+   print("}")
 
 def make_loop(
       op='op',
@@ -269,34 +269,34 @@ def make_loop(
    if if_format:
       src_format = if_format
       dst_format = if_format
-      print interp("if (dst_format == #{dst_format}")
+      print(interp("if (dst_format == #{dst_format}"))
       if texture:
-         print interp("&& src_format == #{src_format}")
-      print ")"
+         print(interp("&& src_format == #{src_format}"))
+      print(")")
    elif copy_format:
       assert opaque and white
-      print interp("if (dst_format == src_format && src_size == #{src_size})")
+      print(interp("if (dst_format == src_format && src_size == #{src_size})"))
 
-   print "{"
+   print("{")
 
    if texture:
-      print """\
+      print("""\
          uint8_t *lock_data = texture->locked_region.data;
          const int src_pitch = texture->locked_region.pitch;
          const al_fixed du_dx = al_ftofix(s->du_dx);
          const al_fixed dv_dx = al_ftofix(s->dv_dx);
-         """
+         """)
 
       if opaque:
          # If texture coordinates never wrap around then we can simplify the
          # innermost loop. It doesn't seem to have so great an impact when the
          # loop is complicated by blending.
-         print """\
+         print("""\
             const float steps = x2 - x1 + 1;
             const float end_u = u + steps * s->du_dx;
             const float end_v = v + steps * s->dv_dx;
             if (end_u >= 0 && end_u < s->w && end_v >= 0 && end_v < s->h) {
-            """
+            """)
          make_innermost_loop(
             op=op,
             src_mode=src_mode,
@@ -312,7 +312,7 @@ def make_loop(
             tiling=False,
             alpha_only=alpha_only
             )
-         print "} else"
+         print("} else")
 
    make_innermost_loop(
       op=op,
@@ -329,7 +329,7 @@ def make_loop(
       alpha_only=alpha_only
       )
 
-   print "}"
+   print("}")
 
 def make_innermost_loop(
       op='op',
@@ -347,61 +347,61 @@ def make_innermost_loop(
       alpha_only=True
       ):
 
-   print "{"
+   print("{")
 
    if texture:
       # In non-tiling mode we can hoist offsets out of the loop.
       if tiling:
-         print """\
+         print("""\
             al_fixed uu = al_ftofix(u);
             al_fixed vv = al_ftofix(v);
             const int uu_ofs = offset_x - texture->lock_x;
             const int vv_ofs = offset_y - texture->lock_y;
             const al_fixed w = al_ftofix(s->w);
             const al_fixed h = al_ftofix(s->h);
-            """
+            """)
          uu_ofs = "uu_ofs"
          vv_ofs = "vv_ofs"
       else:
-         print """\
+         print("""\
             al_fixed uu = al_ftofix(u) + ((offset_x - texture->lock_x) << 16);
             al_fixed vv = al_ftofix(v) + ((offset_y - texture->lock_y) << 16);
-            """
+            """)
          uu_ofs = vv_ofs = "0"
 
-   print "for (; x1 <= x2; x1++) {"
+   print("for (; x1 <= x2; x1++) {")
 
    if not texture:
-      print """\
+      print("""\
          ALLEGRO_COLOR src_color = cur_color;
-         """
+         """)
    else:
-      print interp("""\
+      print(interp("""\
          const int src_x = (uu >> 16) + #{uu_ofs};
          const int src_y = (vv >> 16) + #{vv_ofs};
          uint8_t *src_data = lock_data
             + src_y * src_pitch
             + src_x * #{src_size};
-         """)
+         """))
 
       if copy_format:
          pass
       else:
-         print interp("""\
+         print(interp("""\
             ALLEGRO_COLOR src_color;
             _AL_INLINE_GET_PIXEL(#{src_format}, src_data, src_color, false);
-            """)
+            """))
          if grad:
-            print """\
+            print("""\
             SHADE_COLORS(src_color, cur_color);
-            """
+            """)
          elif not white:
-            print """\
+            print("""\
             SHADE_COLORS(src_color, s->cur_color);
-            """
+            """)
 
    if copy_format:
-      print interp("""\
+      print(interp("""\
          switch (#{src_size}) {
             case 4:
                memcpy(dst_data, src_data, 4);
@@ -419,12 +419,12 @@ def make_innermost_loop(
                *dst_data++ = *src_data;
                break;
          }
-         """)
+         """))
    elif shade:
       blend = "_al_blend_inline"
       if alpha_only:
          blend = "_al_blend_alpha_inline"
-      print interp("""\
+      print(interp("""\
          {
             ALLEGRO_COLOR dst_color;
             ALLEGRO_COLOR result;
@@ -435,20 +435,20 @@ def make_innermost_loop(
                #{const_color}, &result);
             _AL_INLINE_PUT_PIXEL(#{dst_format}, dst_data, result, true);
          }
-         """)
+         """))
    else:
-      print interp("""\
+      print(interp("""\
          _AL_INLINE_PUT_PIXEL(#{dst_format}, dst_data, src_color, true);
-         """)
+         """))
 
    if texture:
-      print """\
+      print("""\
          uu += du_dx;
          vv += dv_dx;
-         """
+         """)
 
       if tiling:
-         print """\
+         print("""\
          if (_AL_EXPECT_FAIL(uu < 0))
             uu += w;
          else if (_AL_EXPECT_FAIL(uu >= w))
@@ -458,22 +458,22 @@ def make_innermost_loop(
             vv += h;
          else if (_AL_EXPECT_FAIL(vv >= h))
             vv -= h;
-         """
+         """)
 
    if grad:
-      print """\
+      print("""\
          cur_color.r += gs->color_dx.r;
          cur_color.g += gs->color_dx.g;
          cur_color.b += gs->color_dx.b;
          cur_color.a += gs->color_dx.a;
-         """
+         """)
 
-   print """\
+   print("""\
       }
-   }"""
+   }""")
 
 if __name__ == "__main__":
-   print """\
+   print("""\
 // Warning: This file was created by make_scanline_drawers.py - do not edit.
 
 #if __GNUC__
@@ -481,7 +481,7 @@ if __name__ == "__main__":
 #else
 #define _AL_EXPECT_FAIL(expr) (expr)
 #endif
-"""
+""")
 
    make_drawer("shader_solid_any_draw_shade")
    make_drawer("shader_solid_any_draw_opaque")
