@@ -177,7 +177,6 @@ ALLEGRO_DISPLAY_INTERFACE *_al_display_gp2xwiz_framebuffer_driver(void)
    gp2xwiz_vt->update_display_region = gp2xwiz_update_display_region_fb;
    gp2xwiz_vt->acknowledge_resize = gp2xwiz_acknowledge_resize_fb;
    gp2xwiz_vt->create_bitmap = NULL;
-   gp2xwiz_vt->create_sub_bitmap = NULL;
    gp2xwiz_vt->get_backbuffer = gp2xwiz_get_backbuffer_fb;
    gp2xwiz_vt->set_target_bitmap = NULL;
    gp2xwiz_vt->is_compatible_bitmap = gp2xwiz_is_compatible_bitmap_fb;
@@ -186,6 +185,8 @@ ALLEGRO_DISPLAY_INTERFACE *_al_display_gp2xwiz_framebuffer_driver(void)
    gp2xwiz_vt->set_window_title = NULL;
    gp2xwiz_vt->set_window_position = NULL;
    gp2xwiz_vt->get_window_position = gp2xwiz_get_window_position_fb;
+   gp2xwiz_vt->set_window_constraints = NULL;
+   gp2xwiz_vt->get_window_constraints = NULL;
    gp2xwiz_vt->set_display_flag = NULL;
    gp2xwiz_vt->wait_for_vsync = gp2xwiz_wait_for_vsync_fb;
 
