@@ -1,3 +1,5 @@
+#error This driver has not been updated to the new mouse API.
+
 /*         ______   ___    ___
  *        /\  _  \ /\_ \  /\_ \
  *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___
@@ -21,10 +23,10 @@
 #include <fcntl.h>
 #include <sys/time.h>
 
-#include "allegro.h"
-#include "allegro/internal/aintern.h"
-#include "allegro/platform/aintunix.h"
-#include "linalleg.h"
+#include "allegro5/allegro.h"
+#include "allegro5/internal/aintern.h"
+#include "allegro5/platform/aintunix.h"
+#include "allegro5/linalleg.h"
 
 
 static int intellimouse;
@@ -51,40 +53,39 @@ static int packet_size;
  *  mouse down or left the data bytes become negative and have ones 
  *  here instead.
  */
-static int processor (unsigned char *buf, int buf_size, struct mouse_info *info)
+static int processor (unsigned char *buf, int buf_size)
 {
-   info->updated = 0;
+   int r, l, m, x, y, z;
+
    if (buf_size < packet_size) return 0;  /* not enough data, spit it out for now */
 
    /* if data is invalid, return no motion and all buttons released */
-   info->l = info->r = info->m = info->x = info->y = info->z = 0;
-   info->updated = 1;
    if (intellimouse) {
       if ((buf[0] & 0xc8) != 0x08) return 1; /* invalid byte, eat it */
    }
    else {
       if ((buf[0] & 0xc0) != 0x00) return 1; /* invalid byte, eat it */
    }
-   
-   /* data is valid, process it */
-   info->l = !!(buf[0] & 1);
-   info->r = !!(buf[0] & 2);
-   info->m = !!(buf[0] & 4);
 
-   info->x = buf[1];
-   info->y = buf[2];
-   if (buf[0] & 0x10) info->x -= 256;
-   if (buf[0] & 0x20) info->y -= 256;
+   /* data is valid, process it */
+   l = !!(buf[0] & 1);
+   r = !!(buf[0] & 2);
+   m = !!(buf[0] & 4);
+
+   x = buf[1];
+   y = buf[2];
+   if (buf[0] & 0x10) x -= 256;
+   if (buf[0] & 0x20) y -= 256;
 
    if (intellimouse) {
-      info->z = buf[3] & 0xf;
-      if (info->z) 
-	 info->z = (info->z-7) >> 3;
+      z = buf[3] & 0xf;
+      if (z) 
+	 z = (z-7) >> 3;
    }
    else
-      info->z = 0;
+      z = 0;
 
-   info->updated = 1;
+   __al_linux_mouse_handler(x, y, z, l+(r<<1)+(m<<2));
    return packet_size; /* yum */
 }
 
@@ -93,7 +94,7 @@ static int processor (unsigned char *buf, int buf_size, struct mouse_info *info)
  *  if there's a reasonable chance that this driver can work with that
  *  data.
  */
-static int analyse_data (AL_CONST char *buffer, int size)
+static int analyse_data (const char *buffer, int size)
 {
 	int pos = 0;
 	int packets = 0, errors = 0;
@@ -169,22 +170,38 @@ static void wakeup_im (int fd)
  */
 static int mouse_init (void)
 {
-	char tmp1[128], tmp2[128], tmp3[128];
-	AL_CONST char *udevice;
+	char tmp1[128], tmp2[128];
+	const char *udevice;
+	int flags;
+
+        static const char * const default_devices[] = {
+		"/dev/mouse",
+		"/dev/input/mice",
+        };
 
 	/* Find the device filename */
 	udevice = get_config_string (uconvert_ascii ("mouse", tmp1),
 				     uconvert_ascii ("mouse_device", tmp2),
-				     uconvert_ascii ("/dev/mouse", tmp3));
+				     NULL);
 
 	/* Open mouse device.  Devices are cool. */
-	if (intellimouse)
-		intdrv.device = open (uconvert_toascii (udevice, tmp1), O_RDWR | O_NONBLOCK);
-	else
-		intdrv.device = open (uconvert_toascii (udevice, tmp1), O_RDONLY | O_NONBLOCK);
+        flags = O_NONBLOCK | (intellimouse ? O_RDWR : O_RDONLY);
+        if (udevice) {
+		intdrv.device = open (uconvert_toascii (udevice, tmp1), flags);
+        }
+        else {
+		size_t n;
+		for (n = 0; n < sizeof(default_devices) / sizeof(default_devices[0]); n++) {
+			intdrv.device = open (default_devices[n], flags);
+			if (intdrv.device >= 0)
+				break;
+		}
+        }
+
 	if (intdrv.device < 0) {
 		uszprintf (allegro_error, ALLEGRO_ERROR_SIZE, get_config_text ("Unable to open %s: %s"),
-			   udevice, ustrerror (errno));
+			   udevice ? udevice : get_config_text("one of the default mice devices"),
+			   ustrerror (errno));
 		return -1;
 	}
 
@@ -203,7 +220,7 @@ static int mouse_init (void)
  */
 static int ps2_mouse_init (void)
 {
-	intellimouse = FALSE;
+	intellimouse = false;
 	packet_size = 3;
     	return mouse_init ();
 }
@@ -213,7 +230,7 @@ static int ps2_mouse_init (void)
  */
 static int ips2_mouse_init (void)
 {
-	intellimouse = TRUE;
+	intellimouse = true;
     	packet_size = 4;
     	return mouse_init ();
 }
@@ -230,8 +247,8 @@ static void mouse_exit (void)
 MOUSE_DRIVER mousedrv_linux_ps2 =
 {
 	MOUSEDRV_LINUX_PS2,
-	empty_string,
-	empty_string,
+	"",
+	"",
 	"Linux PS/2 mouse",
 	ps2_mouse_init,
 	mouse_exit,
@@ -241,14 +258,16 @@ MOUSE_DRIVER mousedrv_linux_ps2 =
 	__al_linux_mouse_set_range,
 	__al_linux_mouse_set_speed,
 	__al_linux_mouse_get_mickeys,
-	analyse_data
+	analyse_data,
+	NULL, /* enable_hardware_cursor */
+	NULL
 };
 
 MOUSE_DRIVER mousedrv_linux_ips2 =
 {
 	MOUSEDRV_LINUX_IPS2,
-	empty_string,
-	empty_string,
+	"",
+	"",
 	"Linux PS/2 Intellimouse",
 	ips2_mouse_init,
 	mouse_exit,
@@ -258,5 +277,7 @@ MOUSE_DRIVER mousedrv_linux_ips2 =
 	__al_linux_mouse_set_range,
 	__al_linux_mouse_set_speed,
 	__al_linux_mouse_get_mickeys,
-	analyse_data
+	analyse_data,
+	NULL, /* enable_hardware_cursor */
+	NULL
 };
