@@ -129,6 +129,7 @@ int _al_ogl_get_glformat(int format, int component)
       glformats[ALLEGRO_PIXEL_FORMAT_SINGLE_CHANNEL_8][2] = GL_RED;
    }
    #else
+   // TODO: Check supported formats by various GLES versions
    static const int glformats[ALLEGRO_NUM_PIXEL_FORMATS][3] = {
       /* Skip pseudo formats */
       {0, 0, 0},
@@ -182,8 +183,10 @@ char const *_al_gl_error_string(GLenum e)
       ERR(GL_INVALID_ENUM)
       ERR(GL_INVALID_VALUE)
       ERR(GL_INVALID_OPERATION)
+#ifdef ALLEGRO_CFG_OPENGL_FIXED_FUNCTION
       ERR(GL_STACK_OVERFLOW)
       ERR(GL_STACK_UNDERFLOW)
+#endif
       ERR(GL_OUT_OF_MEMORY)
 #ifdef ALLEGRO_CFG_OPENGL_PROGRAMMABLE_PIPELINE
       ERR(GL_INVALID_FRAMEBUFFER_OPERATION)
@@ -461,12 +464,14 @@ static bool ogl_upload_bitmap(ALLEGRO_BITMAP *bitmap)
          post_generate_mipmap = true;
       }
       else {
+#ifdef ALLEGRO_CFG_OPENGL_FIXED_FUNCTION
          glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
           e = glGetError();
           if (e) {
               ALLEGRO_ERROR("glTexParameteri for texture %d failed (%s).\n",
                             ogl_bitmap->texture, _al_gl_error_string(e));
           }
+#endif
       }
    }
 
@@ -562,15 +567,21 @@ static void ogl_destroy_bitmap(ALLEGRO_BITMAP *bitmap)
 {
    ALLEGRO_BITMAP_EXTRA_OPENGL *ogl_bitmap = bitmap->extra;
    ALLEGRO_DISPLAY *disp;
+   ALLEGRO_DISPLAY *bmp_disp;
    ALLEGRO_DISPLAY *old_disp = NULL;
 
    ASSERT(!al_is_sub_bitmap(bitmap));
 
+   bmp_disp = _al_get_bitmap_display(bitmap);
    disp = al_get_current_display();
-   if (_al_get_bitmap_display(bitmap)->ogl_extras->is_shared == false &&
-       _al_get_bitmap_display(bitmap) != disp) {
+   if (bmp_disp->ogl_extras->is_shared == false &&
+       bmp_disp != disp) {
       old_disp = disp;
-      _al_set_current_display_only(_al_get_bitmap_display(bitmap));
+      _al_set_current_display_only(bmp_disp);
+   }
+
+   if (bmp_disp->ogl_extras->opengl_target == bitmap) {
+     bmp_disp->ogl_extras->opengl_target = NULL;
    }
 
    al_remove_opengl_fbo(bitmap);
@@ -583,7 +594,7 @@ static void ogl_destroy_bitmap(ALLEGRO_BITMAP *bitmap)
    if (old_disp) {
       _al_set_current_display_only(old_disp);
    }
-   
+
    al_free(ogl_bitmap);
 }
 
@@ -1033,8 +1044,7 @@ ALLEGRO_BITMAP *_al_ogl_create_bitmap(ALLEGRO_DISPLAY *d, int w, int h,
       }
    }
 
-   /* Android included because some devices require POT FBOs */
-   if (IS_OPENGLES || !d->extra_settings.settings[ALLEGRO_SUPPORT_NPOT_BITMAP]) {
+   if (!d->extra_settings.settings[ALLEGRO_SUPPORT_NPOT_BITMAP]) {
       true_w = pot(true_w);
       true_h = pot(true_h);
    }
