@@ -1,3 +1,17 @@
+/*         ______   ___    ___
+ *        /\  _  \ /\_ \  /\_ \
+ *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___
+ *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
+ *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
+ *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
+ *            \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/
+ *                                           /\____/
+ *                                           \_/__/
+ *
+ *      SDL keyboard driver.
+ *
+ *      See LICENSE.txt for copyright information.
+ */
 #include "allegro5/allegro.h"
 #include "allegro5/internal/aintern_system.h"
 #include "allegro5/platform/allegro_internal_sdl.h"
@@ -60,7 +74,9 @@ void _al_sdl_keyboard_event(SDL_Event *e)
       event.keyboard.keycode = keyboard->table[e->key.keysym.scancode];
       event.keyboard.unichar = keyboard->unicode[e->key.keysym.scancode];
       event.keyboard.display = _al_sdl_find_display(e->key.windowID);
-      _al_event_source_emit_event(es, &event);
+      if (!e->key.repeat) {
+         _al_event_source_emit_event(es, &event);
+      }
 
       if (keyboard->create_extra_char[e->key.keysym.scancode]) {
          event.keyboard.type = ALLEGRO_EVENT_KEY_CHAR;
@@ -156,8 +172,8 @@ static bool sdl_init_keyboard(void)
    ADD_SAME(BACKSLASH, '\\');
    add(SDL_SCANCODE_NONUSHASH, 0, '#');
    ADD_SAME(SEMICOLON, ';');
-   add(SDL_SCANCODE_APOSTROPHE, 0, '\'');
-   add(SDL_SCANCODE_GRAVE, 0, 0);
+   add(SDL_SCANCODE_APOSTROPHE, ALLEGRO_KEY_QUOTE, '\'');
+   add(SDL_SCANCODE_GRAVE, ALLEGRO_KEY_TILDE, '~');
    ADD_SAME(COMMA, ',');
    add(SDL_SCANCODE_PERIOD, ALLEGRO_KEY_FULLSTOP, '.');
    ADD_SAME(SLASH, '/');
@@ -363,6 +379,7 @@ static ALLEGRO_KEYBOARD *sdl_get_keyboard(void)
 
 static bool sdl_set_keyboard_leds(int leds)
 {
+   (void)leds;
    return false;
 }
 
