@@ -21,6 +21,7 @@ ALLEGRO_DEBUG_CHANNEL("SDL")
 
 static ALLEGRO_SYSTEM_INTERFACE *vt;
 
+#ifdef DEBUGMODE
 #define _E(x) if (type == x) return #x;
 static char const *event_name(int type)
 {
@@ -68,6 +69,7 @@ static char const *event_name(int type)
    return "(unknown)";
 }
 #undef _E
+#endif
 
 static void sdl_heartbeat(void)
 {
@@ -342,6 +344,9 @@ ALLEGRO_SYSTEM_INTERFACE *_al_sdl_system_driver(void)
    vt->close_library = sdl_close_library;*/
    vt->heartbeat = sdl_heartbeat;
    vt->heartbeat_init = sdl_heartbeat_init;
+   vt->get_time = _al_sdl_get_time;
+   vt->rest = _al_sdl_rest;
+   vt->init_timeout = _al_sdl_init_timeout;
 
    return vt;
 }
