@@ -109,7 +109,10 @@ static ALLEGRO_PATH *early_get_exename_path(void)
 #elif defined(ALLEGRO_ANDROID)
    return _al_android_get_path(ALLEGRO_EXENAME_PATH);
 #elif defined(ALLEGRO_SDL)
-   return al_create_path_for_directory(SDL_GetBasePath());
+   char* p = SDL_GetBasePath();
+   ALLEGRO_PATH *path = al_create_path_for_directory(p);
+   SDL_free(p);
+   return path;
 #else
    #error early_get_exename_path not implemented
 #endif
@@ -352,6 +355,13 @@ ALLEGRO_SYSTEM *al_get_system_driver(void)
    return active_sysdrv;
 }
 
+/* Function: al_get_system_id
+ */
+ALLEGRO_SYSTEM_ID al_get_system_id(void)
+{
+   ASSERT(active_sysdrv);
+   return active_sysdrv->vt->id;
+}
 
 /* Function: al_get_system_config
  */
@@ -485,4 +495,38 @@ void _al_close_library(void *library)
       active_sysdrv->vt->close_library(library);
 }
 
+
+/* Function: al_get_time
+ */
+double al_get_time(void)
+{
+   ASSERT(active_sysdrv);
+
+   if (active_sysdrv->vt->get_time)
+      return active_sysdrv->vt->get_time();
+   return 0.0;
+}
+
+
+/* Function: al_rest
+ */
+void al_rest(double seconds)
+{
+   ASSERT(active_sysdrv);
+
+   if (active_sysdrv->vt->rest)
+      active_sysdrv->vt->rest(seconds);
+}
+
+
+/* Function: al_init_timeout
+ */
+void al_init_timeout(ALLEGRO_TIMEOUT *timeout, double seconds)
+{
+   ASSERT(active_sysdrv);
+
+   if (active_sysdrv->vt->init_timeout)
+      active_sysdrv->vt->init_timeout(timeout, seconds);
+}
+
 /* vim: set sts=3 sw=3 et: */
