@@ -698,7 +698,8 @@ void _al_triangle_2d(ALLEGRO_BITMAP* texture, ALLEGRO_VERTEX* v1, ALLEGRO_VERTEX
    v2c = v2->color;
    v3c = v3->color;
 
-   al_get_separate_blender(&op, &src_mode, &dst_mode, &op_alpha, &src_alpha, &dst_alpha);
+   al_get_separate_bitmap_blender(&op,
+      &src_mode, &dst_mode, &op_alpha, &src_alpha, &dst_alpha);
    if (_AL_DEST_IS_ZERO && _AL_SRC_NOT_MODIFIED) {
       shade = 0;
    }
@@ -829,7 +830,8 @@ void _al_draw_soft_triangle(
       min_y = clip_min_y;
 
    if (al_is_bitmap_locked(target)) {
-      if (!bitmap_region_is_locked(target, min_x, min_y, max_x - min_x, max_y - min_y))
+      if (!bitmap_region_is_locked(target, min_x, min_y, max_x - min_x, max_y - min_y) ||
+          _al_pixel_format_is_video_only(target->locked_region.format))
          return;
    } else {
       if (!(lr = al_lock_bitmap_region(target, min_x, min_y, max_x - min_x, max_y - min_y, ALLEGRO_PIXEL_FORMAT_ANY, 0)))
