@@ -29,4 +29,8 @@
 
 _AL_BEGIN_JOYSTICK_DRIVER_LIST
 _AL_JOYSTICK_DRIVER_DIRECTX
+#ifdef ALLEGRO_CFG_XINPUT 
+_AL_JOYSTICK_DRIVER_XINPUT
+_AL_JOYSTICK_DRIVER_WINDOWS_ALL
+#endif
 _AL_END_JOYSTICK_DRIVER_LIST
