@@ -40,6 +40,11 @@
 #include "allegro5/internal/aintern_xkeyboard.h"
 #include "allegro5/internal/aintern_xsystem.h"
 
+#ifdef ALLEGRO_RASPBERRYPI
+#include "allegro5/internal/aintern_raspberrypi.h"
+#define ALLEGRO_SYSTEM_XGLX ALLEGRO_SYSTEM_RASPBERRYPI
+#endif
+
 ALLEGRO_DEBUG_CHANNEL("keyboard")
 
 /*----------------------------------------------------------------------*/
@@ -54,14 +59,21 @@ typedef struct ALLEGRO_KEYBOARD_XWIN
 {
    ALLEGRO_KEYBOARD parent;
    ALLEGRO_KEYBOARD_STATE state;
+   // Quit if Ctrl-Alt-Del is pressed.
+   bool three_finger_flag;
 } ALLEGRO_KEYBOARD_XWIN;
 
 
+typedef struct ALLEGRO_KEY_REPEAT_DATA {
+   XKeyEvent *event;
+   bool found;
+} ALLEGRO_KEY_REPEAT_DATA;
+
 
 /* the one and only keyboard object */
 static ALLEGRO_KEYBOARD_XWIN the_keyboard;
 
-
+static int last_press_code = -1;
 
 #ifdef ALLEGRO_XWINDOWS_WITH_XIM
 static XIM xim = NULL;
@@ -357,6 +369,22 @@ static int find_unknown_key_assignment(int i)
 }
 
 
+/* XCheckIfAny predicate that checks if this event may be a key repeat event */
+static Bool check_for_repeat(Display *display, XEvent *event, XPointer arg)
+{
+   ALLEGRO_KEY_REPEAT_DATA *d = (ALLEGRO_KEY_REPEAT_DATA *)arg;
+
+   (void)display;
+
+   if (event->type == KeyPress &&
+      event->xkey.keycode == d->event->keycode &&
+      (event->xkey.time - d->event->time) < 4) {
+      d->found = true;
+   }
+
+   return False;
+}
+
 
 /* _al_xwin_keyboard_handler:
  *  Keyboard "interrupt" handler.
@@ -421,22 +449,21 @@ void _al_xwin_keyboard_handler(XKeyEvent *event, ALLEGRO_DISPLAY *display)
    }
    else { /* Key release. */
      /* HACK:
-      * Detect key repeat by looking forward to see if this release
-      * is followed directly by a press event, in which case we assume
-      * that this release event was generated in response to that key
-      * press event. Events are simultaneous if they are separated by
-      * less than 4 ms (a value that worked well on one machine where
-      * this hack was needed).
-      * 
-      * This is unnecessary on systems where XkbSetDetectableAutorepeat
-      * works.
+      * Detect key repeat by looking forward to see if this release is
+      * followed by a press event, in which case we assume that this release
+      * event was generated in response to that key press event. Events are
+      * simultaneous if they are separated by less than 4 ms (a value that
+      * worked well on one machine where this hack was needed).
+      *
+      * This is unnecessary on systems where XkbSetDetectableAutorepeat works.
       */
       if (XPending(event->display) > 0) {
-         XEvent next_event;
-         XPeekEvent(event->display, &next_event);
-         if ((next_event.type == KeyPress) &&
-             (next_event.xkey.keycode == event->keycode) &&
-             (next_event.xkey.time - event->time) < 4) {
+         ALLEGRO_KEY_REPEAT_DATA d;
+         XEvent dummy;
+         d.event = event;
+         d.found = false;
+         XCheckIfEvent(event->display, &dummy, check_for_repeat, (XPointer)&d);
+         if (d.found) {
             return;
          }
       }
@@ -520,6 +547,7 @@ static bool _al_xwin_get_keyboard_mapping(void)
    int i;
    int count;
    int missing = 0;
+
    ALLEGRO_SYSTEM_XGLX *system = (void *)al_get_system_driver();
 
    memset(used, 0, sizeof used);
@@ -624,8 +652,8 @@ static bool _al_xwin_get_keyboard_mapping(void)
     * For normal use, a user never should have to touch [xkeymap] anymore
     * though, and proper written programs will not hardcode such mappings.
     */
-   ALLEGRO_CONFIG *c = system->system.config;
-   
+   ALLEGRO_CONFIG *c = al_get_system_config();
+
    char const *key;
    ALLEGRO_CONFIG_ENTRY *it;
    key = al_get_first_config_entry(c, "xkeymap", &it);
@@ -701,14 +729,14 @@ static int x_keyboard_init(void)
    memcpy(key_names, _al_keyboard_common_names, sizeof key_names);
 
    _al_mutex_lock(&s->lock);
-   
-/* HACK: XkbSetDetectableAutoRepeat is broken in some versions of X.Org
+
+   /* HACK: XkbSetDetectableAutoRepeat is broken in some versions of X.Org */
    Bool supported;
    XkbSetDetectableAutoRepeat(s->x11display, True, &supported);
    if (!supported) {
       ALLEGRO_WARN("XkbSetDetectableAutoRepeat failed.\n");
    }
-*/
+
 #ifdef ALLEGRO_XWINDOWS_WITH_XIM
    ALLEGRO_INFO("Using X Input Method.\n");
 
@@ -726,12 +754,13 @@ static int x_keyboard_init(void)
       ALLEGRO_WARN("Could not set default locale.\n");
    }
 
-/* TODO: is this needed?
-   modifiers = XSetLocaleModifiers("@im=none");
+   /* By default never use an input method as we are not prepared to
+    * handle any of them. This is still enough to get composed keys.
+    */
+   char const *modifiers = XSetLocaleModifiers("@im=none");
    if (modifiers == NULL) {
       ALLEGRO_WARN("XSetLocaleModifiers failed.\n");
    }
-*/
 
    xim = XOpenIM(s->x11display, NULL, NULL, NULL);
    if (xim == NULL) {
@@ -812,7 +841,7 @@ static void x_keyboard_exit(void)
    if (!xkeyboard_installed)
       return;
    xkeyboard_installed = 0;
-   
+
    ALLEGRO_SYSTEM_XGLX *s = (void *)al_get_system_driver();
 
    _al_mutex_lock(&s->lock);
@@ -861,6 +890,7 @@ static ALLEGRO_KEYBOARD *xkeybd_get_keyboard(void);
 static bool xkeybd_set_keyboard_leds(int leds);
 static const char *xkeybd_keycode_to_name(int keycode);
 static void xkeybd_get_keyboard_state(ALLEGRO_KEYBOARD_STATE *ret_state);
+static void xkeybd_clear_keyboard_state(void);
 
 
 
@@ -878,7 +908,8 @@ static ALLEGRO_KEYBOARD_DRIVER keydrv_xwin =
    xkeybd_get_keyboard,
    xkeybd_set_keyboard_leds,
    xkeybd_keycode_to_name,
-   xkeybd_get_keyboard_state
+   xkeybd_get_keyboard_state,
+   xkeybd_clear_keyboard_state
 };
 
 
@@ -902,8 +933,18 @@ static bool xkeybd_init_keyboard(void)
 
    _al_event_source_init(&the_keyboard.parent.es);
 
+   the_keyboard.three_finger_flag = true;
+
+   const char *value = al_get_config_value(al_get_system_config(),
+         "keyboard", "enable_three_finger_exit");
+   if (value) {
+      the_keyboard.three_finger_flag = !strncmp(value, "true", 4);
+   }
+   ALLEGRO_DEBUG("Three finger flag enabled: %s\n",
+      the_keyboard.three_finger_flag ? "true" : "false");
+
    //_xwin_keydrv_set_leds(_key_shifts);
-   
+
    /* Get the pid, which we use for the three finger salute */
    main_pid = getpid();
 
@@ -969,12 +1010,25 @@ static void xkeybd_get_keyboard_state(ALLEGRO_KEYBOARD_STATE *ret_state)
 
 
 
+/* xkeybd_get_keyboard_state:
+ *  Clear the current keyboard state, with any necessary locking.
+ */
+static void xkeybd_clear_keyboard_state(void)
+{
+   _al_event_source_lock(&the_keyboard.parent.es);
+   {
+      last_press_code = -1;
+      memset(&the_keyboard.state, 0, sizeof(the_keyboard.state));
+   }
+   _al_event_source_unlock(&the_keyboard.parent.es);
+}
+
+
+
 /* handle_key_press: [bgman thread]
  *  Hook for the X event dispatcher to handle key presses.
  *  The caller must lock the X-display.
  */
-static int last_press_code = -1;
-
 static void handle_key_press(int mycode, int unichar, int filtered,
    unsigned int modifiers, ALLEGRO_DISPLAY *display)
 {
@@ -1020,11 +1074,14 @@ static void handle_key_press(int mycode, int unichar, int filtered,
    }
    _al_event_source_unlock(&the_keyboard.parent.es);
 
+// FIXME?
+#ifndef ALLEGRO_RASPBERRYPI
    /* Toggle mouse grab key.  The system driver should not be locked here. */
    if (last_press_code && !is_repeat) {
       ALLEGRO_SYSTEM_XGLX *system = (void *)al_get_system_driver();
-      if (system->toggle_mouse_grab_keycode == mycode &&
-         (modifiers & system->toggle_mouse_grab_modifiers)
+      if (system->toggle_mouse_grab_keycode &&
+          system->toggle_mouse_grab_keycode == mycode &&
+          (modifiers & system->toggle_mouse_grab_modifiers)
             == system->toggle_mouse_grab_modifiers)
       {
          if (system->mouse_grab_display == display)
@@ -1033,9 +1090,10 @@ static void handle_key_press(int mycode, int unichar, int filtered,
             al_grab_mouse(display);
       }
    }
+#endif
 
    /* Exit by Ctrl-Alt-End.  */
-   if ((_al_three_finger_flag)
+   if ((the_keyboard.three_finger_flag)
        && ((mycode == ALLEGRO_KEY_DELETE) || (mycode == ALLEGRO_KEY_END))
        && (modifiers & ALLEGRO_KEYMOD_CTRL)
        && (modifiers & (ALLEGRO_KEYMOD_ALT | ALLEGRO_KEYMOD_ALTGR)))
@@ -1077,4 +1135,5 @@ static void handle_key_release(int mycode, ALLEGRO_DISPLAY *display)
    _al_event_source_unlock(&the_keyboard.parent.es);
 }
 
+
 /* vim: set sts=3 sw=3 et: */
