@@ -20,7 +20,7 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <allegro.h>
+#include <allegro5.h>
 #include <math.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -117,6 +117,7 @@ static int scale_icon(ICON_DATA *icon)
    int i_x, i_y;
    float k, f_x, f_y, a, b, r1, r2;
    float red, green, blue, alpha;
+   unsigned char *p;
    unsigned int color;
    
    if (icon->original->w > icon->original->h) {
@@ -292,7 +293,7 @@ static void usage(void)
 
 
 
-static int copy_file(AL_CONST char *filename, AL_CONST char *dest_path)
+static int copy_file(const char *filename, const char *dest_path)
 {
    char *buffer = NULL;
    char dest_file[1024];
@@ -676,4 +677,3 @@ exit_error_bundle:
    rmdir(bundle_contents_macos_dir);
    goto exit_error;
 }
-END_OF_MAIN()
